@@ -5,6 +5,93 @@
 #ifndef _PKCS11N_H_
 #define _PKCS11N_H_
 
+/* "friendly" macros to allow us to deprecate certain #defines */
+#if defined(__GNUC__) && (__GNUC__ > 3) && !defined(NSS_SKIP_DEPRECATION_WARNING)
+/* make GCC warn when we use these #defines */
+/*
+ *  This is really painful because GCC doesn't allow us to mark random
+ *  #defines as deprecated. We can only mark the following:
+ *      functions, variables, and types.
+ *  const variables will create extra storage for everyone including this
+ *       header file, so it's undesirable.
+ *  functions could be inlined to prevent storage creation, but will fail
+ *       when constant values are expected (like switch statements).
+ *  enum types do not seem to pay attention to the deprecated attribute.
+ *
+ *  That leaves typedefs. We declare new types that we then deprecate, then
+ *  cast the resulting value to the deprecated type in the #define, thus
+ *  producting the warning when the #define is used.
+ *
+ *  To make this work with the C preprocessor, we first include a
+ *  _NSS_DEPRECATE_DEFINE_TYPE(type, name, message) declaration in our code
+ *  with:
+ *      'type'    being the base type that the #define is for (CK_TRUST,
+ *                CK_MECHANISM_TTYPE, CK_KEY_TYPE etc.)
+ *      'name'    being a unique name for the type (usually the name of the
+ *                #define)
+ *      'message' is the string to print out when the compilier warns about
+ *                the deprecated object. This only works in GCC >= 4.5 and
+ *                is ignored on all other platforms.
+ *  We then do a normal #define with _NSS_DEPRECATED_DEFINE_TYPE(name, value)
+ *  as the value with:
+ *      'name'    the same unique name used in _NSS_DEPRECATE_DEFINE_TYPE
+ *      'value'   what you would normally place in the value.
+ *
+ * Just deprecating structs are easier, if the struct isn't already aliased
+ * to some other struct, make your own alias for it, then use
+ *
+ * _NSS_DEPRECATE_STRUCT(alias, name, message)
+ * with:
+ *    'alias'   the alias structure.
+ *    'name'    the name of the deprecated structure
+ *    'message' is the string to print out when the compilier warns about
+ *              the deprecated object. This only works in GCC >= 4.5 and
+ *              is ignored on all other platforms.
+ *
+ *
+ */
+#if (__GNUC__ == 4) && (__GNUC_MINOR__ < 5)
+/* The mac doesn't like the friendlier deprecate messages. I'm assuming this
+ * is a gcc version issue rather than mac or ppc specific */
+#define _NSS_DEPRECATE_DEFINE_TYPE(type, name, message) \
+    typedef type __deprecate_##name __attribute__((deprecated));
+#define _NSS_DEPRECATE_DEFINE_VALUE(name, value) ((__deprecate_##name)(value))
+
+#define _NSS_DEPRECATE_STRUCT(alias, name, message) \
+    typedef alias name __attribute__((deprecated));
+
+#else /*__GNUC__ >= 4.5 */
+
+#define _NSS_DEPRECATE_DEFINE_TYPE(type, name, message) \
+    typedef type __deprecate_##name __attribute__((deprecated(message)));
+#define _NSS_DEPRECATE_DEFINE_VALUE(name, value) ((__deprecate_##name)(value))
+
+#define _NSS_DEPRECATE_STRUCT(alias, name, message) \
+    typedef alias name __attribute__((deprecated(message)));
+
+#endif /* __GNUC__ < 4.5 */
+#else  /* !__GNUC__ */
+#if defined(_WIN32) && !defined(NSS_SKIP_DEPRECATION_WARNING)
+/* windows, we just use pragma deprecated(identifier) */
+
+#define _NSS_DEPRECATE_DEFINE_TYPE(type, name, message)
+/* #pragma deprecated(name) */
+#define _NSS_DEPRECATE_DEFINE_VALUE(name, value) (value)
+
+#define _NSS_DEPRECATE_STRUCT(alias, name, message) \
+    /* #pragma deprecated(name) */ typedef alias name;
+
+#else /* not WIN or GNUC, just define the structure */
+
+/* fall back to just defining the thing we want without a deprecation warning */
+#define _NSS_DEPRECATE_DEFINE_TYPE(type, name, message)
+#define _NSS_DEPRECATE_DEFINE_VALUE(name, value) (value)
+#define _NSS_DEPRECATE_STRUCT(alias, name, message) \
+    typedef alias name;
+
+#endif /* !_WIN32 */
+#endif /*!__GNUC__ */
+
 /*
  * pkcs11n.h
  *
@@ -115,34 +202,77 @@
 /*
  * Trust attributes:
  *
- * If trust goes standard, these probably will too.  So I'll
- * put them all in one place.
+ * If trust attributes are now standard, but we didn't use
+ * NSS specific names, so the CKA_ names collide with the standard
+ * names. We'll update NSS to use specific names, and applications
+ * can use The #NSS_USE_STANDARD_TRUST define to select which values
+ * the CKA_TRUST_XXX names should map to.
+ *
+ * In our code we'll expect CKA_NSS_TRUST_xxx attributes in
+ * CKO_NSS_TRUST objects and CKA_PKCS_TRUST attributes in
+ * CKO_TRUST objects.
  */
-
-#define CKA_TRUST (CKA_NSS + 0x2000)
+#define CKA_NSS_TRUST_BASE (CKA_NSS + 0x2000)
 
 /* "Usage" key information */
-#define CKA_TRUST_DIGITAL_SIGNATURE (CKA_TRUST + 1)
-#define CKA_TRUST_NON_REPUDIATION (CKA_TRUST + 2)
-#define CKA_TRUST_KEY_ENCIPHERMENT (CKA_TRUST + 3)
-#define CKA_TRUST_DATA_ENCIPHERMENT (CKA_TRUST + 4)
-#define CKA_TRUST_KEY_AGREEMENT (CKA_TRUST + 5)
-#define CKA_TRUST_KEY_CERT_SIGN (CKA_TRUST + 6)
-#define CKA_TRUST_CRL_SIGN (CKA_TRUST + 7)
+#define CKA_NSS_TRUST_DIGITAL_SIGNATURE (CKA_NSS_TRUST_BASE + 1)
+#define CKA_NSS_TRUST_NON_REPUDIATION (CKA_NSS_TRUST_BASE + 2)
+#define CKA_NSS_TRUST_KEY_ENCIPHERMENT (CKA_NSS_TRUST_BASE + 3)
+#define CKA_NSS_TRUST_DATA_ENCIPHERMENT (CKA_NSS_TRUST_BASE + 4)
+#define CKA_NSS_TRUST_KEY_AGREEMENT (CKA_NSS_TRUST_BASE + 5)
+#define CKA_NSS_TRUST_KEY_CERT_SIGN (CKA_NSS_TRUST_BASE + 6)
+#define CKA_NSS_TRUST_CRL_SIGN (CKA_NSS_TRUST_BASE + 7)
 
 /* "Purpose" trust information */
-#define CKA_TRUST_SERVER_AUTH (CKA_TRUST + 8)
-#define CKA_TRUST_CLIENT_AUTH (CKA_TRUST + 9)
-#define CKA_TRUST_CODE_SIGNING (CKA_TRUST + 10)
-#define CKA_TRUST_EMAIL_PROTECTION (CKA_TRUST + 11)
-#define CKA_TRUST_IPSEC_END_SYSTEM (CKA_TRUST + 12)
-#define CKA_TRUST_IPSEC_TUNNEL (CKA_TRUST + 13)
-#define CKA_TRUST_IPSEC_USER (CKA_TRUST + 14)
-#define CKA_TRUST_TIME_STAMPING (CKA_TRUST + 15)
-#define CKA_TRUST_STEP_UP_APPROVED (CKA_TRUST + 16)
+#define CKA_NSS_TRUST_SERVER_AUTH (CKA_NSS_TRUST_BASE + 8)
+#define CKA_NSS_TRUST_CLIENT_AUTH (CKA_NSS_TRUST_BASE + 9)
+#define CKA_NSS_TRUST_CODE_SIGNING (CKA_NSS_TRUST_BASE + 10)
+#define CKA_NSS_TRUST_EMAIL_PROTECTION (CKA_NSS_TRUST_BASE + 11)
+#define CKA_NSS_TRUST_IPSEC_END_SYSTEM (CKA_NSS_TRUST_BASE + 12)
+#define CKA_NSS_TRUST_IPSEC_TUNNEL (CKA_NSS_TRUST_BASE + 13)
+#define CKA_NSS_TRUST_IPSEC_USER (CKA_NSS_TRUST_BASE + 14)
+#define CKA_NSS_TRUST_TIME_STAMPING (CKA_NSS_TRUST_BASE + 15)
+#define CKA_NSS_TRUST_STEP_UP_APPROVED (CKA_NSS_TRUST_BASE + 16)
 
-#define CKA_CERT_SHA1_HASH (CKA_TRUST + 100)
-#define CKA_CERT_MD5_HASH (CKA_TRUST + 101)
+#define CKA_NSS_CERT_SHA1_HASH (CKA_NSS_TRUST_BASE + 100)
+#define CKA_NSS_CERT_MD5_HASH (CKA_NSS_TRUST_BASE + 101)
+
+#ifdef NSS_USE_STANDARD_TRUST
+/* Names take on the PKCS #11 standard values */
+#define CKA_TRUST_SERVER_AUTH CKA_PKCS_TRUST_SERVER_AUTH
+#define CKA_TRUST_CLIENT_AUTH CKA_PKCS_TRUST_CLIENT_AUTH
+#define CKA_TRUST_CODE_SIGNING CKA_PKCS_TRUST_CODE_SIGNING
+#define CKA_TRUST_EMAIL_PROTECTION CKA_PKCS_TRUST_EMAIL_PROTECTION
+#define CKA_TRUST_TIME_STAMPING CKA_PKCS_TRUST_TIME_STAMPING
+#define CKA_TRUST_OCSP_SIGNING CKA_PKCS_TRUST_OCSP_SIGNING
+#else
+/* Names take on the legacy NSS values */
+/* NOTE these don't actually colide with the PKCS #11 standard values
+ * but we want to rename to with the NSS in them anyway. When
+ * you set NSS_USE_STANDARD_TRUST, the non _NSS_ names will
+ * go away */
+#define CKA_TRUST CKA_NSS_TRUST_BASE
+#define CKA_TRUST_DIGITAL_SIGNATURE CKA_NSS_TRUST_DIGITAL_SIGNATURE
+#define CKA_TRUST_NON_REPUDIATION CKA_NSS_TRUST_NON_REPUDIATION
+#define CKA_TRUST_KEY_ENCIPHERMENT CKA_NSS_TRUST_KEY_ENCIPHERMENT
+#define CKA_TRUST_DATA_ENCIPHERMENT CKA_NSS_TRUST_DATA_ENCIPHERMENT
+#define CKA_TRUST_KEY_AGREEMENT CKA_NSS_TRUST_KEY_AGREEMENT
+#define CKA_TRUST_KEY_CERT_SIGN CKA_NSS_TRUST_KEY_CERT_SIGN
+#define CKA_TRUST_CRL_SIGN CKA_NSS_TRUST_CRL_SIGN
+#define CKA_TRUST_EMAIL_PROTECTION CKA_NSS_TRUST_EMAIL_PROTECTION
+#define CKA_TRUST_IPSEC_END_SYSTEM CKA_NSS_TRUST_IPSEC_END_SYSTEM
+#define CKA_TRUST_IPSEC_TUNNEL CKA_NSS_TRUST_IPSEC_TUNNEL
+#define CKA_TRUST_IPSEC_USER CKA_NSS_TRUST_IPSEC_USER
+#define CKA_TRUST_STEP_UP_APPROVED CKA_NSS_TRUST_STEP_UP_APPROVED
+#define CKA_CERT_SHA1_HASH CKA_NSS_CERT_SHA1_HASH
+#define CKA_CERT_MD5_HASH CKA_NSS_CERT_MD5_HASH
+
+/* These names collide with pkcs #11 standard names */
+#define CKA_TRUST_SERVER_AUTH CKA_NSS_TRUST_SERVER_AUTH
+#define CKA_TRUST_CLIENT_AUTH CKA_NSS_TRUST_CLIENT_AUTH
+#define CKA_TRUST_CODE_SIGNING CKA_NSS_TRUST_CODE_SIGNING
+#define CKA_TRUST_TIME_STAMPING CKA_NSS_TRUST_TIME_STAMPING
+#endif
 
 /* NSS trust stuff */
 
@@ -250,7 +380,7 @@
 
 #define CKM_NSS_CHACHA20_CTR (CKM_NSS + 33)
 
-/* IKE mechanism (to be proposed to PKCS #11 */
+/* IKE mechanisms now defined in PKCS #11, use those instead now */
 #define CKM_NSS_IKE_PRF_PLUS_DERIVE (CKM_NSS + 34)
 #define CKM_NSS_IKE_PRF_DERIVE (CKM_NSS + 35)
 #define CKM_NSS_IKE1_PRF_DERIVE (CKM_NSS + 36)
@@ -411,90 +541,31 @@ typedef struct CK_NSS_HKDFParams {
 /*
  * CK_NSS_IKE_PRF_PLUS_PARAMS is a structure that provides the parameters to
  * the CKM_NSS_IKE_PRF_PLUS_DERIVE mechanism.
- * The fields of the structure have the following meanings:
- *      prfMechanism    underlying MAC mechanism used to generate the prf.
- *      bHasSeedKey     hSeed key is present.
- *      hSeedKey        optional seed from key
- *      pSeedData       optional seed from data.
- *      ulSeedDataLen   length of optional seed data.
- *        If no seed data is present this value is NULL.
+ * It is now standardized, so The struct is just an alias for the standard
+ * struct in pkcs11t.h.
  */
-typedef struct CK_NSS_IKE_PRF_PLUS_DERIVE_PARAMS {
-    CK_MECHANISM_TYPE prfMechanism;
-    CK_BBOOL bHasSeedKey;
-    CK_OBJECT_HANDLE hSeedKey;
-    CK_BYTE_PTR pSeedData;
-    CK_ULONG ulSeedDataLen;
-} CK_NSS_IKE_PRF_PLUS_DERIVE_PARAMS;
+typedef struct CK_IKE2_PRF_PLUS_DERIVE_PARAMS CK_NSS_IKE_PRF_PLUS_DERIVE_PARAMS;
 
 /* CK_NSS_IKE_PRF_DERIVE_PARAMS is a structure that provides the parameters to
- *  the CKM_NSS_IKE_PRF_DERIVE mechanism.
- *
- * The fields of the structure have the following meanings:
- *     prfMechanism underlying MAC mechanism used to generate the prf.
- *     bRekey       hNewKey is present.
- *     pNi          Ni value
- *     ulNiLen      length of Ni
- *     pNr          Nr value
- *     ulNrLen      length of Nr
- *     hNewKey      New key value to drive the rekey.
+ * the CKM_NSS_IKE_PRF_DERIVE mechanism.
+ * It is now standardized, so The struct is just an alias for the standard
+ * struct in pkcs11t.h.
  */
-typedef struct CK_NSS_IKE_PRF_DERIVE_PARAMS {
-    CK_MECHANISM_TYPE prfMechanism;
-    CK_BBOOL bDataAsKey;
-    CK_BBOOL bRekey;
-    CK_BYTE_PTR pNi;
-    CK_ULONG ulNiLen;
-    CK_BYTE_PTR pNr;
-    CK_ULONG ulNrLen;
-    CK_OBJECT_HANDLE hNewKey;
-} CK_NSS_IKE_PRF_DERIVE_PARAMS;
+typedef struct CK_IKE_PRF_DERIVE_PARAMS CK_NSS_IKE_PRF_DERIVE_PARAMS;
 
 /* CK_NSS_IKE1_PRF_DERIVE_PARAMS is a structure that provides the parameters
  * to the CKM_NSS_IKE_PRF_DERIVE mechanism.
- *
- * The fields of the structure have the following meanings:
- *     prfMechanism  underlying MAC mechanism used to generate the prf.
- *     bRekey        hNewKey is present.
- *     pCKYi         CKYi value
- *     ulCKYiLen     length of CKYi
- *     pCKYr         CKYr value
- *     ulCKYrLen     length of CKYr
- *     hNewKey       New key value to drive the rekey.
+ * It is now standardized, so The struct is just an alias for the standard
+ * struct in pkcs11t.h.
  */
-typedef struct CK_NSS_IKE1_PRF_DERIVE_PARAMS {
-    CK_MECHANISM_TYPE prfMechanism;
-    CK_BBOOL bHasPrevKey;
-    CK_OBJECT_HANDLE hKeygxy;
-    CK_OBJECT_HANDLE hPrevKey;
-    CK_BYTE_PTR pCKYi;
-    CK_ULONG ulCKYiLen;
-    CK_BYTE_PTR pCKYr;
-    CK_ULONG ulCKYrLen;
-    CK_BYTE keyNumber;
-} CK_NSS_IKE1_PRF_DERIVE_PARAMS;
+typedef struct CK_IKE1_PRF_DERIVE_PARAMS CK_NSS_IKE1_PRF_DERIVE_PARAMS;
 
 /* CK_NSS_IKE1_APP_B_PRF_DERIVE_PARAMS is a structure that provides the
  * parameters to the CKM_NSS_IKE_APP_B_PRF_DERIVE mechanism.
- *
- * The fields of the structure have the following meanings:
- *     prfMechanism  underlying MAC mechanism used to generate the prf.
- *     bHasKeygxy    hKeygxy exists
- *     hKeygxy       optional key to hash in the prf
- *     pExtraData    optional extra data to hash in the prf
- *     ulExtraData   length of the optional extra data.
- *
- * CK_NSS_IKE_APP_B_PRF_DERIVE can take wither CK_NSS_IKE1_APP_B_PRF_DRIVE_PARAMS
- * or a single CK_MECHANISM_TYPE. In the latter cases bHashKeygx is assumed to
- * be false and ulExtraDataLen is assumed to be '0'.
+ * It is now standardized, so The struct is just an alias for the standard
+ * struct in pkcs11t.h.
  */
-typedef struct CK_NSS_IKE1_APP_B_PRF_DERIVE_PARAMS {
-    CK_MECHANISM_TYPE prfMechanism;
-    CK_BBOOL bHasKeygxy;
-    CK_OBJECT_HANDLE hKeygxy;
-    CK_BYTE_PTR pExtraData;
-    CK_ULONG ulExtraDataLen;
-} CK_NSS_IKE1_APP_B_PRF_DERIVE_PARAMS;
+typedef struct CK_IKE1_EXTENDED_DERIVE_PARAMS CK_NSS_IKE1_APP_B_PRF_DERIVE_PARAMS;
 
 /*
  * Parameter for the TLS extended master secret key derivation mechanisms:
@@ -523,15 +594,10 @@ typedef struct CK_NSS_TLS_EXTENDED_MASTER_KEY_DERIVE_PARAMS {
 /*
  * Trust info
  *
- * This isn't part of the Cryptoki standard (yet), so I'm putting
- * all the definitions here.  Some of this would move to nssckt.h
- * if trust info were made part of the standard.  In view of this
- * possibility, I'm putting my (NSS) values in the NSS
- * vendor space, like everything else.
+ * This now part of the Cryptoki standard , These are all the
+ * old vendor defined symbols.
  */
 
-typedef CK_ULONG CK_TRUST;
-
 /* The following trust types are defined: */
 #define CKT_VENDOR_DEFINED 0x80000000
 
@@ -555,52 +621,18 @@ typedef CK_ULONG CK_TRUST;
  * labels have never been accurate to what was really implemented.
  * The new labels correctly reflect what the values effectively mean.
  */
-#if defined(__GNUC__) && (__GNUC__ > 3)
-/* make GCC warn when we use these #defines */
-/*
- *  This is really painful because GCC doesn't allow us to mark random
- *  #defines as deprecated. We can only mark the following:
- *      functions, variables, and types.
- *  const variables will create extra storage for everyone including this
- *       header file, so it's undesirable.
- *  functions could be inlined to prevent storage creation, but will fail
- *       when constant values are expected (like switch statements).
- *  enum types do not seem to pay attention to the deprecated attribute.
- *
- *  That leaves typedefs. We declare new types that we then deprecate, then
- *  cast the resulting value to the deprecated type in the #define, thus
- *  producting the warning when the #define is used.
- */
-#if (__GNUC__ == 4) && (__GNUC_MINOR__ < 5)
-/* The mac doesn't like the friendlier deprecate messages. I'm assuming this
- * is a gcc version issue rather than mac or ppc specific */
-typedef CK_TRUST __CKT_NSS_UNTRUSTED __attribute__((deprecated));
-typedef CK_TRUST __CKT_NSS_VALID __attribute__((deprecated));
-typedef CK_TRUST __CKT_NSS_MUST_VERIFY __attribute__((deprecated));
-#else
-/* when possible, get a full deprecation warning. This works on gcc 4.5
- * it may work on earlier versions of gcc */
-typedef CK_TRUST __CKT_NSS_UNTRUSTED __attribute__((deprecated("CKT_NSS_UNTRUSTED really means CKT_NSS_MUST_VERIFY_TRUST")));
-typedef CK_TRUST __CKT_NSS_VALID __attribute__((deprecated("CKT_NSS_VALID really means CKT_NSS_NOT_TRUSTED")));
-typedef CK_TRUST __CKT_NSS_MUST_VERIFY __attribute__((deprecated("CKT_NSS_MUST_VERIFY really functions as CKT_NSS_TRUST_UNKNOWN")));
-#endif
-#define CKT_NSS_UNTRUSTED ((__CKT_NSS_UNTRUSTED)CKT_NSS_MUST_VERIFY_TRUST)
-#define CKT_NSS_VALID ((__CKT_NSS_VALID)CKT_NSS_NOT_TRUSTED)
-/* keep the old value for compatibility reasons*/
-#define CKT_NSS_MUST_VERIFY ((__CKT_NSS_MUST_VERIFY)(CKT_NSS + 4))
-#else
-#ifdef _WIN32
-/* This magic gets the windows compiler to give us a deprecation
- * warning */
-#pragma deprecated(CKT_NSS_UNTRUSTED, CKT_NSS_MUST_VERIFY, CKT_NSS_VALID)
-#endif
-/* CKT_NSS_UNTRUSTED really means CKT_NSS_MUST_VERIFY_TRUST */
-#define CKT_NSS_UNTRUSTED CKT_NSS_MUST_VERIFY_TRUST
-/* CKT_NSS_VALID really means CKT_NSS_NOT_TRUSTED */
-#define CKT_NSS_VALID CKT_NSS_NOT_TRUSTED
-/* CKT_NSS_MUST_VERIFY was always treated as CKT_NSS_TRUST_UNKNOWN */
-#define CKT_NSS_MUST_VERIFY (CKT_NSS + 4) /*really means trust unknown*/
-#endif
+_NSS_DEPRECATE_DEFINE_TYPE(CK_TRUST, CKT_NSS_UNTRUSTED,
+                           "CKT_NSS_UNTRUSTED really means CKT_NSS_MUST_VERIFY_TRUST")
+#define CKT_NSS_UNTRUSTED \
+    _NSS_DEPRECATE_DEFINE_VALUE(CKT_NSS_UNTRUSTED, CKT_NSS_MUST_VERIFY_TRUST)
+_NSS_DEPRECATE_DEFINE_TYPE(CK_TRUST, CKT_NSS_VALID,
+                           "CKT_NSS_VALID really means CKT_NSS_NOT_TRUSTED")
+#define CKT_NSS_VALID \
+    _NSS_DEPRECATE_DEFINE_VALUE(CKT_NSS_VALID, CKT_NSS_NOT_TRUSTED)
+_NSS_DEPRECATE_DEFINE_TYPE(CK_TRUST, CKT_NSS_MUST_VERIFY,
+                           "CKT_NSS_MUST_VERIFY really functions as CKT_NSS_TRUST_UNKNOWN");
+#define CKT_NSS_MUST_VERIFY \
+    _NSS_DEPRECATE_DEFINE_VALUE(CKT_NSS_MUST_VERIFY, CKT_NSS_TRUST_UNKNOWN)
 
 /*
  * These are not really PKCS #11 values specifically. They are the 'loadable'
